## Adding the library to an existing project

This is the structure of a classic android project:

```
|-Yourapp
|--.idea
|--build.gradle <- important
|--gradle
|--app
|----build
|----libs <- add the aar file here
|----jars
|----src
|----build.gradle <- important
```

1. from the root of your project add the physiloglib-release.arr to `app/libs`. Create the `libs` folder if needed.
2. Open the `build.gradle` found in the **root** of your project, and add `flatDir{dirs 'libs'}` like below

        allprojects {
            repositories {
                jcenter()

                //Add this
                flatDir {
                    dirs 'libs'
                }
            }
        }

3. Open the `build.gradle` found in the **app** folder of your project, and add the library to your dependencies

        dependencies {
            
            //Add this
            compile(name:'physiloglib-release', ext:'aar')
        }




-----------------------

  

The Physilog Android Library allows the creation of new Android applications that use Physilog 5 devices or the integration of those devices into existing applications. 



## Usage of PhysilogLib

In addition to the explanations given here, three complete examples of interactions are provided in the sample app.

 * SensorFragment: Display the raw data from the physilogs it's connected to.  
 * FileFragment: Start the recording of measurement in a file and display its transfer to the android device.    
 * ConnectionFragment: Do a manual bluetooth scan and automatically connect to one of the available physilog device.   




### PhysilogManager  


The state of the library is maintained in a [Singleton](https://en.wikipedia.org/wiki/Singleton_pattern) called `PhysilogManager` that handle the connection with the physilogs, and give methods to access them. To get an instance of PhysilogManager use `PhysilogManager.getInstance(context);`  

From the physilogManager it is then possible to get an instance of the `Physilog` class (or the list of all of them). This object encapsulate the state of a physilog device and give functions that allows to interact with it. Below is a simple example:  

  

```java

PhysilogManager pm = PhysilogManage.getInstance(this);
List<Physilog> physilogs = pm.getPhysilogs();

String TAG = "Demo";

//Log some information
for(Physilog ph: physilogs){
    Log.d(TAG, "name: " +  ph.getName());
    Log.d(TAG, "battery: " + ph.getBatteryLevel());
    Log.d(TAG, "state: " + ph.getStateText());
    ...
}

//start streaming of data
for(Physilog ph: physilogs){
    if(ph.getState() == Physilog.STATE_STANDBY_OFF){
        //information transmitted by streaming can be received using broadcast (see next section)
        ph.startStreaming();
    }
}
```


### Android Permissions

PhysilogLib require multiple permissions to work, it needs bluetooth access, localization access (for bluetooth on versions of Android > 5) and storage read/write to store the measurements. Since PhysilogLib is an Android library, it has already declared the needed permissions in its own AndroidManifest.xml.

**Tip**: In Android studio you can see what the combined manifest will look like by pressing the `Merged Manifest` button on the bottom when editing your manifest.

But since Android 6, it is now required to dynamically request each permissions, so when your first use the `PhysilogManager` in you code, you need to request the permissions:

```java
//If you are in an activity and not a fragment, replace both getter by "this"
physilogManager = PhysilogManager.getInstance(getContext());
physilogManager.checkAndRequestPermissions(getActivity());
```



-----------------------



## Physilog(s) Connections and Disconnections


To do the bluetooth connections between the Android devices and the Physilogs. The library offers you two choices: Use a pre-made interface that handle all the bluetooth connections and that can be integrated into any activity or displayed on it's own. Or use a set of functions to manually start bluetooth scan, connect to a physilog, .. to build your own interface the way you want.


### AvailablePhysilogsFragment

This is the pre-made interface. It's an Android [Fragment](https://developer.android.com/guide/components/fragments.html). that can be incorporated in any activty or displayed temporarily on the screen to let the user choose to which physilogs to connect to. It's used in the first 2 examples of the sample app (SensorFragment and FileFragment) If you want to incorporate into the layout of another activity you can declare it like this:

```xml
<fragment android:name="com.gaitup.lib.physilog.AvailablePhysilogsFragment"
    android:id="@+id/available_physilogs_fragment"
    android:layout_width="320dp"
    android:layout_height="match_parent"
    android:layout_alignParentTop="true"
    android:layout_alignParentStart="true" />
```

And you don't have to care about anything, it will do its job on it's own. We will see a little bit later how you will be informed of new connections, disconnections, ...

You can also do more fancy things. For example if you use the sample app on a phone-sized device, the fragment will not be part of the layout but only appear when a button on the upper right is pressed. This is done through the use of the Fragment manager. The code for this can be seen in the MainActivity.java.


### Manual Physilog (dis)connection (See ConnectionFragment.java)

If you don't want to use the pre-made `AvailablePhysilogsFragment` you can use the methods provided by the `PhysilogManager` to create the same functionnality. PhysilogManager has a function `scanForPhysilogs(...)` that start a bluetooth scan for available physilogs. Once it's finished it calls a listener that has the class `AvailblePhysilogListener` and give it the list of found physilogs. Let's look at this example:

```java

    private Physilog physilog;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        ...
        ...
        physilogManager = PhysilogManager.getInstance(getContext());
        physilogManager.checkAndRequestPermissions(getActivity());

        //start searching for physilogs
        physilogManager.scanForPhysilogs(physilogListener);
    }


    //Called when the scan for Physilogs has finished
    private AvailblePhysilogListener physilogListener = new AvailblePhysilogListener() {
        @Override
        public void availablePhysilogs(List<Physilog> physilogs) {

            //List all possible
            for (Physilog ph : physilogs){
                Log.d("Physilog", "Found this available: " + ph.getName())
            }

            //Connect to the first of the list
            if (physilogs.size() > 0){
                physilog = physilogs.get(0);
                physilogManager.connectToPhysilog(physilog.getDevice());
            }

        }
    };

    @Override
    public void onDestroy() {
        //Don't forget to disconect
        if (physilog != null && physilog.getConnectionState() == ConnectionState.Connected){
            physilogManager.disconnectFromPhysilog(physilog.getDevice());
        }
    }

```

As you can see in the example two other functions are usefeul `connectToPhysilog(BluetoothDevice)` and `disconnectFromPhysilog(BluetoothDevice)` are to be used for connection and disconnection.


**Note**: Bluetooth device are typically only discoverable for a short period of time. So it is possible that between the time of the bluetooth scan and the call of the function `connectToPhysilog`, the physilog has stopped receiving connections, or has alrady connected with another device. So you will need to keep track of the connected devices. (We will see how a little bit later)




-----------------------



### Physilog's states.

A `Physilog` instance has two states. One for the bluetooth connection (connected, connecting, ...) and one for the actual state of the physilog device (recording, streaming, ...).


##### Connection State

This is the state of the bluetooth connection between the Android device and the physilog device. It's defined as an java enumeration and can be accessed through the `getConnectionState()` method of a physilog instance and can be one of the following:


```
public enum ConnectionState {
  Disconnected, Disconnecting, Connecting, Connected
}
```

Each of the state also has a member variable called `textId` that point to a text in the library's string.xml. You can use this to display or log the connection state.

**Tip**: Since it's very common to only work on Connected physilogs, the physilogManager has a function `getConnectedPhysilogs` that directly return the list of all connected physilogs.


##### Device State

This is the state of the physilog's device, in the sample app you can see it displayed in the list of available physilogs. It comes directly from the physical device and is represented as a byte. To facilitate it's usage several constants are defined in the Physilog class:


```java
class Physilog {

    public final static byte STATE_SET_PHYSILOG_HW;
    public final static byte STATE_STANDBY_OFF;
    public final static byte STATE_PHYSILOG_ON_FILE;
    public final static byte STATE_PHYSILOG_PAUSED;
    public final static byte STATE_USB_MASSE_STORAGE;
    public final static byte STATE_CONFIGURE_PHYSILOG;
    public final static byte STATE_ENABLE_BLE;
    public final static byte STATE_DISABLE_BLE;
    public final static byte STATE_WORKING_LAST_FILE;
    public final static byte STATE_PHYSILOG_ON_STREAM;

    ....
 }
```

As you can see in the sample app's source code, it is usually used to verify that a physilog device is in Standby state before starting to record / stream. To get a textual representation of this state use the `getStateText()` method from a physilog object. A quick example on how to use this state:

```java
boolean allReady = true;
for(Physilog ph: physilogManager.getConnectedPhysilogs()){
    if(ph.getState() != Physilog.STATE_STANDBY_OFF){
        allReady = false;
        break;
    }
}

if (allReady){
    //Use the physilogs here
}        
```


-----------------------




### Broadcasts

The primary way for the library to send information to the application is through local [broadcasts](https://developer.android.com/guide/components/broadcasts.html). Activity/Fragments wanting to be informed of new connections of a physilog, new sensor's data, ... must register for the broadcast using the code below:  

```java
    @Override
    protected void onCreate(Bundle savedInstanceState) {
    	...
    	...
    	LocalBroadcastManager bm = LocalBroadcastManager.getInstance(this);

    	bm.registerReceiver(broadcastReceiverNewPhysilogConnected, new IntentFilter("new_physilog_connected"));
        bm.registerReceiver(broadcastReceiverPhysilogDisconnected, new IntentFilter("physilog_disconnected"));
    }

    @Override
    public void onDestroy() {
        ...
        ...
    	LocalBroadcastManager bm = LocalBroadcastManager.getInstance(this);

        bm.unregisterReceiver(broadcastReceiverNewPhysilogConnected);
        bm.unregisterReceiver(broadcastReceiverPhysilogDisconnected);
    }
```


Where `broadcastReceiverNewPhysilogConnected` and `broadcastReceiverPhysilogDisconnected` are BroadcastReceiver called when the specified broadcast is triggered. From the intent they receive they can get the information provided by the broadcast. For example when a  new physilog device is connected:  


```java
    BroadcastReceiver broadcastReceiverNewPhysilogConnected = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle b = intent.getExtras();
            final BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");

            Log.d("Physilog", "A new Physilog is connected: " + device.getName());
        }
    };
```

A list of all the available local broadcast can be found at the end of this document.  



-----------------------



### OnResume

In addition to registering in onCreate and unregistering in onDestroy, the Activities and Fragments **must** call the library when onResume is called (see the [Activity Life-cycle graph](https://developer.android.com/guide/components/activities/activity-lifecycle.html#alc)). This allows the library to refresh the connection status with all the physilogs and re-connnect with the one who lost connection during the application's pause.  


```java
    @Override
    public void onResume() {
        super.onResume();
        physilogManager.initActivity();
    }
```



-----------------------


### Streaming sensor's data  (See SensorFragment.java)


The get the sensor's data of a physilog, it must be in streaming mode. To do this you only need to call the method `startStreaming()` on the physilog if it is in Standby State (As shown in an example above). A Physilog can not be streaming and at the same time recording to a file. Once the physilog is in streaming mode, the data will automatically arrive trough the broadcasts. 

There are 2 broadcasts that bring this information:

* new_sensors_data:
	* "BluetoothDevice": BluetoothDevice
	* "SensorsValues": float[8]
        * [0]: <reserved>
		* [1]: gyroX
		* [2]: gyroY
		* [3]: gyroZ
        * [4]: acceleroX
        * [5]: acceleroY
        * [6]: acceleroZ
        * [7]: Baro

* new_quaternion_data:
    * "BluetoothDevice": BluetoothDevice
    * "QuaternionValues": float[4]
        * [0]: w
        * [1]: x
        * [2]: y
        * [3]: z


This is an example of how the BroadcastReceiver would look for the first broadcast.

```java

BroadcastReceiver broadcastReceiverNewSensorsData = new BroadcastReceiver() {
    @Override
    public void onReceive(Context context, Intent intent) {
  
        Bundle b = intent.getExtras();
        final BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");
        final float[] values =  b.getFloatArray("SensorsValues");

        float time = values[0];
        float gyroX = values[1];
        float gyroY = values[2];
        ...
};
```


### File transfer (See FileFragment.java)


To have the physilog record the measurements to a file and send it over bluetooth is done through another set of broadcasts. This time the setup is a bit more complicated. Before calling `startRecording()` on the physilogs, we must first call a method that will prepare the physilogs for the recording and create a file for the data to go:

```java
//Start the recording for every physilog connected
physilogManager.prepareForRecording();

for(Physilog ph: physilogManager.getConnectedPhysilogs()){
	ph.startRecording();
	//Update the ui
}
```

This time there are 3 broadcasts related:  


* file_name:
	* "BluetoothDevice": BluetoothDevice
	* "name": String

* file_stream_progress:
	* "BluetoothDevice": BluetoothDevice
	* "progress": int

* file_generated:
	* "BluetoothDevice": BluetoothDevice
	* "progress": int


The full path of the file (not just the name) can be obtained through the `getFilePath()` method of a physilog object


##### Folder

By default the generated file is recorded on the primary extern storage of the android device in folder called "Physilog". It is possible to choose in which folder they will arrive:

```java

//Choose where the files will be recorded
String dir = Environment.getExternalStorageDirectory().toString() + "/Example";
physilogManager.setFilesDirectory(dir);
```

This will not move the already existing files previously stored. This will need to be done manually with a file explorer.


-----------------------


## List of broadcasts

Here is a list of all available broadcasts and for each of them the extra information they bring.

* new_physilog_connected:
	* "BluetoothDevice": BluetoothDevice

* physilog_disconnected:
	* "BluetoothDevice": BluetoothDevice

* new_sensors_data:
	* "BluetoothDevice": BluetoothDevice
	* "SensorsValues": float[8]
        * [0]: <reserved>
		* [1]: gyroX
		* [2]: gyroY
		* [3]: gyroZ
        * [4]: acceleroX
        * [5]: acceleroY
        * [6]: acceleroZ
        * [7]: Baro

* new_quaternion_data:
	* "BluetoothDevice": BluetoothDevice
	* "QuaternionValues": float[4]
        * [0]: w
        * [1]: x
        * [2]: y
        * [3]: z

* state_changed:
	* "BluetoothDevice": BluetoothDevice
	* "new_state": byte

* state_read:
	* "BluetoothDevice": BluetoothDevice
	* "StateValue": byte

* sensors_settings_value:
	* "BluetoothDevice": BluetoothDevice
	* "AccFreq": int
	* "GyroFreq": int
	* "BaroFreq": int
	* "AccG": int
	* "GyroG": int

* battery_level:
	* "BluetoothDevice": BluetoothDevice
	* "value": byte // from 0 to 100 and -1 for invalid battery information

* file_name:
	* "BluetoothDevice": BluetoothDevice
	* "name": String

* file_stream_progress:
	* "BluetoothDevice": BluetoothDevice
	* "progress": int //from 0 to 100

* file_generated:
	* "BluetoothDevice": BluetoothDevice
	* "progress": int //Should be 100 since the file is finished
